import java.util.Arrays;
import java.util.*;
   
private String[][] rawdata;
private String[][] subCatNames;
private HashMap<String,Integer> adjList;   
private String[][] catNames;  
private float[][] percentages;
private int[] totalAll;
private int[] totalSelf;

private String data_filename = "spl_data_no_comics.csv";
private String names_filename = "dewey_names.csv";
   
   
 int numCat;
 int numSeries;
 float graphSize=230;
 float epsilon=0.001;
 float[][] values;
 float[] hues;
 Series[] series;
 int transp=70;
 boolean sorted;
 
 PFont font;
PFont fontLarge;
PFont fontItalic;
 
CategoryGroup myCatGroup; 
 

void setup(){
  smooth();
  size(1200,600);  
  colorMode(HSB,360,100,100,100);
  
  catNames=importData(names_filename);
  numCat=catNames.length;
  numSeries=catNames.length;
  rawdata=importData(data_filename);
  adjList= getAdjList(rawdata);

  percentages=getPercentageData(adjList);
  

   font=loadFont("HelveticaNeue-Light-14.vlw");
  fontLarge=loadFont("HelveticaNeue-Bold-36.vlw");
  fontItalic =loadFont("HelveticaNeue-LightItalic-12.vlw");

  
generate();
    }
    
void generate(){
  sorted=false;
  
  hues=new float[numSeries];
  series= new Series[numSeries];
  myCatGroup = new CategoryGroup(numSeries,graphSize);
  //fill in series hues array
    for(int i=0;i<numSeries;i++){
      String seriesName=catNames[i][1];
      String deweyID=catNames[i][0];
      hues[i]=360*float(i)/numSeries;
      color myColor= color(hues[i],60,100,transp);
      float[] insertValues=new float[numCat];
       for (int j=0;j<numCat;j++){
    
      insertValues[j]=percentages[i][j];
       }
       int max=max(totalAll);
      Series mySeries= new Series(seriesName,insertValues,myColor,i,totalSelf[i],totalAll[i],max);
      series[i]=mySeries;
      myCatGroup.addItem(i,mySeries.myCategory);
    }
    
}

    

void regenerate(){
 generate();
  
}


void draw(){
  background(80);
//noLoop();
 //draw lines
 textFont(font,12);
 strokeWeight(0.5);
 stroke(180);
 pushMatrix();
 translate(2*width/3,height/2);
 drawGrid(3);
  
 for(int i=0; i<numSeries;i++){
   series[i].setSize(graphSize);
   series[i].display();
 }
 myCatGroup.display();
 
 popMatrix();
 textAlign(LEFT,TOP);
 pushStyle();
 textFont(fontLarge,32);
 fill(360);
 text("Frequently Borrowed Together", 20,20);
 popStyle();
  for(int i=0;i<numSeries;i++){
  series[i].myLegend.display(240,150+30*i);
  }
 
  textAlign(RIGHT);

 if(sorted){
 text("sorted on category "+series[0].compareBy, width-50,height-50);
 } 
 drawLegendKey();
}

void drawLegendKey(){
  
  pushStyle();
  textFont(fontItalic,12);
  fill(360,70);
  ellipse(200,150+30*11,20,20);
  ellipse(200+70,150+30*11,20,20);
  
 
  fill(80);
  ellipse(200+70,150+30*11,12,12);
  fill(360,50);
  rect(340-20-150,150+30*12-15/2.0,150,15);
  fill(360,70);
  rect(340-20-70,150+30*12-15/2.0,70,15);
  stroke(360,70);
  line(200+70-50,150+30*13,200+70-10,150+30*13);
  noStroke();
  ellipse(200+70,150+30*13,20,20);
  
  
  
  fill(360);
  textFont(fontItalic,12);
  textAlign(LEFT, TOP);
  String intro="The following diagram shows how frequently books from different subject areas were checked out together. Each color on the spider diagram corresponds to a subject area (key below). The axes represent the percent of books that were checked out along with books from other categoies.";
  text(intro, 20,60,400,300);
  textFont(fontItalic,12);
  textAlign(LEFT, CENTER);
   text("Show", 200+20,150+30*11);
   text("Hide", 200+70+20,150+30*11);
   text("Click to Sort", 200+70+20,150+30*13);
  textAlign(LEFT, TOP);
   text("With Same Category",340-10,150+30*12-15/2.0+5); 
  textAlign(RIGHT, TOP);
   text("With Other Categories",340-20-150-10,150+30*12-15/2.0+5);
  fill(360,60);
  text("10%",2*width/3,height/2+graphSize*(1/3.0)+5);
  text("20%",2*width/3,height/2+graphSize*(2/3.0)+5);
  text("30%",2*width/3,height/2+graphSize+5);
  popStyle();
}

void drawGrid(int numTics){
    int l=0;
 for(float theta=0;theta<2*PI*(1-epsilon);theta+=2*PI/numCat){
   float x=graphSize*(1.05)*cos(theta);
   float y=graphSize*(1.05)*sin(theta);
   line(0,0,x,y);
   l++;
 }
 
  //circles
  noFill();
  for(float i=1;i<=numTics;i++){
    ellipse(0,0,2*graphSize*i/numTics,2*graphSize*i/numTics);
  }
   
 }

void randomSort(){
   sortOnCat(int(random(0,numCat))); 
}

void sortOnCat(int cat){
   for(int i=0;i<numSeries;i++){
   series[i].setCompareBy(cat);
   
   }
   Arrays.sort(series);
}

void mousePressed(){ 
  if(myCatGroup.checkSelected()){
     sortOnCat(myCatGroup.selectedItem); 
     sorted=true;
   }
   for (int i=0;i<numSeries;i++){
   if(series[i].myLegend.checkDist()){
    series[i].myLegend.changeState();
  }
   }

  }


void keyPressed(){
  if(keyCode == UP){
    numSeries+=1;
    regenerate();
  }
  /*
    if(keyCode == DOWN){
       if(numSeries>0)
    numSeries-=1;
    regenerate();
  }
    if(keyCode == LEFT){
      if(numCat>0)
    numCat-=1;
    regenerate();
  }
     if(keyCode == RIGHT){
    numCat+=1;
    regenerate();
  }
  
       if(keyCode == SHIFT){
    randomSort();
    sorted=true;
    */
  }
  if(key == 's') save("mockup_XXXX.jpg");
  
}
